# -*- coding: utf-8 -*-

import datetime

from django.conf import settings


class SuspensionMixin(object):
    if 'jmb.suspension' in settings.INSTALLED_APPS:
        def get_stops(self, date_s=datetime.date.today()):
            """
            restituisce tutte le sospensioni dell'Anagrafica alla data specificata
            """
            stop = self.suspensions.filter(date_begin__lte=date_s, date_end__gte=date_s)
            return stop

        def is_stopped(self, date_s=datetime.date.today()):
            """
            restituisce True o False se il Cliente e' sospeso nel giorno specificato
            """
            stop = self.suspensions.filter(date_begin__lte=date_s, date_end__gte=date_s)[0]
            if stop:
                return True
            return False

        def is_stopped_in(self, date_begin=datetime.date.today(), date_end=datetime.date.today()):
            """
            restituisce True o False se eiste già una sospensione dell'Anagrafica nel periodo specificato.
            """
            stop = self.suspensions.filter(date_begin__lte=date_end, date_end__gte=date_begin)[0]
            if stop:
                return True
            return False

        def is_forever_stopped(self, date_s=datetime.date.today()):
            """
            restituisce True o False se il Cliente e' sospeso definitivamente:
            e' sospeso definitivamente se non esiste la data di fine
            """
            stop = self.suspensions.filter(date_begin__lte=date_s, date_end__isnull=True)[0]
            if stop:
                return True
            return False

        def set_start(self, cause, date_end=datetime.date.today()):
            """
            crea una sospensione con data inizio = NULL e data fine = giorno precedente a quello impostato
            L'anagrafica è attiva dal giorno impostato
            """
            from suspension.models import Suspension
            suspension = Suspension(
                organization=self,
                disable_cause=cause,
                date_begin=None,
                date_end=date_end + datetime.timedelta(days=-1)
            )
            suspension.save()

        def set_restart(self, date_end=datetime.date.today()):
            """
            attiva l'Anagrafica disattivata da una sospensione modificando la
            data di fine sospensione inserendo il giorno precedente.
            La data passata deve essere compresa nell'intervallo di sospensione.
            """
            suspensions = self.get_stops(date_end)
            for suspension in suspensions:
                suspension.date_end = date_end + datetime.timedelta(days=-1)
                suspension.save()

        def set_stop(self, cause, date_begin=datetime.date.today(), date_end=None):
            """
            disattiva il cliente creando una sospensione dalla data di inizio
            alla data di fine specificate inserendo una causale.
            Se la data di fine è NULL l'Anagrafica è sospesa definitivamente.
            Non verifica se c'è una sovrapposizione di sospensioni
            CONTROLLI:
            - La data di inizio deve essere <= alla data di fine
            - La data di inizio deve essere > della data di ultima consegna (get_last_ddt())
            """
            if not cause:
                return
            if date_end and date_begin > date_end:
                return
            from ddt.utils import get_last_ddt
            last_ddt = get_last_ddt()
            if date_begin <= last_ddt.date_d:
                return
            from suspension.models import Suspension
            suspension = Suspension(organization=self, disable_cause=cause, date_begin=date_begin, date_end=date_end)
            suspension.save()
