#! -*- coding: utf-8 -*-

from django import forms
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import User
from django.contrib.auth.forms import AuthenticationForm, PasswordResetForm, SetPasswordForm, PasswordChangeForm
from thux.core.middleware.thread_local import get_current_user
from thux.organization.models import Organization, Contact
from thux.organization.utils import send_recovery_password_email, send_reset_password_email, check_password_strenght
from captcha.fields import CaptchaField


class LoginForm(AuthenticationForm):
    def __init__(self, *args, **kwargs):
        super(LoginForm, self).__init__(*args, **kwargs)


class OrganizationRegistrationForm(forms.ModelForm):
    username = forms.CharField(
        max_length=30,
        label=_("nickname"),
        required=True,
    )
    first_name = forms.CharField(
        label=_("first name"),
        required=True
    )
    last_name = forms.CharField(
        label=_("last name"),
        required=True
    )
    email = forms.EmailField(
        label=_("email"),
        required=True
    )
    password1 = forms.CharField(
        widget=forms.PasswordInput(render_value=False),
        label=_("password"),
        required=True
    )
    password2 = forms.CharField(
        widget=forms.PasswordInput(render_value=False),
        label=_("confirm password"),
        required=True
    )
    captcha = CaptchaField()

    class Meta:
        model = Organization
        fields = ('name', 'ssn', 'vat_number', 'url', 'email', 'phone', 'fax',)

    def clean_email(self):
        users = User.objects.filter(
            email__iexact=self.cleaned_data['email']
        )
        if users:
            raise forms.ValidationError(_("email already used by another account"))
        return self.cleaned_data['email']

    def clean_username(self):
        user = get_current_user()
        users = User.objects.filter(
            username__iexact=self.cleaned_data['username']
        )
        if not user.is_anonymous():
            users = users.exclude(pk=user.pk)
        if users:
            raise forms.ValidationError(_("nickname already used by an other account"))
        return self.cleaned_data['username']

    # def clean_password1(self):
    #     score = check_password(self.cleaned_data.get('password1'))[0]
    #     if score > 1:
    #         return self.cleaned_data['password1']
    #     raise forms.ValidationError(_("""
    #         password too simple or too short. make it longer or try some combination with
    #         capitals and simple letters, numbers and symbols.
    #     """))

    def clean(self):
        password1 = self.cleaned_data.get('password1')
        password2 = self.cleaned_data.get('password2')
        first_name = self.cleaned_data.get('first_name')
        last_name = self.cleaned_data.get('last_name')
        email = self.cleaned_data.get('email')
        username = self.cleaned_data.get('username')
        account = None
        if email:
            account = email.split('@')[0]

        msg = None
        if password1 and first_name and password1.lower() == first_name.lower():
            msg = _("password must be different to your first name")
        elif password1 and last_name and password1.lower() == last_name.lower():
            msg = _("password must be different to your last name")
        elif password1 and email and password1.lower() == email.lower():
            msg = _("password must be different to your email")
        elif password1 and account and password1.lower() == account.lower():
            msg = _("password can't derive from parts of your email")
        elif password1 and username and password1.lower() == username.lower():
            msg = _("password must be different to your nickname")
        if msg:
            self._errors["password1"] = self.error_class([msg])

        if password1 and password2 and password1 != password2:
            msg = _("submitted passwords doesn't match")
            self._errors["password2"] = self.error_class([msg])
            del self.cleaned_data["password1"]
            del self.cleaned_data["password2"]
        return self.cleaned_data

    def save(self, commit=True):
        # Creo nuovo utente
        new_user = User.objects.create_user(
            username=self.cleaned_data['username'],
            email=self.cleaned_data['email'],
            password=self.cleaned_data['password1'],
        )
        new_user.first_name = self.cleaned_data['first_name']
        new_user.last_name = self.cleaned_data['last_name']
        new_user.is_active = False
        new_user.save()

        # Creo organization
        organization = super(OrganizationRegistrationForm, self).save(commit=False)
        organization.save()
        return new_user


class ContactRegistrationForm(forms.ModelForm):
    username = forms.CharField(
        max_length=30,
        label=_("nickname"),
        required=True
    )
    first_name = forms.CharField(
        label=_("first name"),
        required=True
    )
    last_name = forms.CharField(
        label=_("last name"),
        required=True
    )
    email = forms.EmailField(
        label=_("email"),
        required=True
    )
    password1 = forms.CharField(
        widget=forms.PasswordInput(render_value=False),
        label=_("password"),
        required=True
    )
    password2 = forms.CharField(
        widget=forms.PasswordInput(render_value=False),
        label=_("confirm password"),
        required=True
    )
    captcha = CaptchaField()

    class Meta:
        model = Contact
        fields = ('title', 'role', 'first_name', 'last_name', 'phone', 'mobile', 'email')

    def clean_email(self):
        users = User.objects.filter(
            email__iexact=self.cleaned_data['email']
        )
        if users:
            raise forms.ValidationError(_("email used by another account"))
        return self.cleaned_data['email']

    def clean_username(self):
        user = get_current_user()
        users = User.objects.filter(
            username__iexact=self.cleaned_data['username']
        )
        if not user.is_anonymous():
            users = users.exclude(pk=user.pk)
        if users:
            raise forms.ValidationError(_("nickname used by another account"))
        return self.cleaned_data['username']

    #    def clean_password1(self):
    #        score = check_password(self.cleaned_data.get('password1'))[0]
    #        if score > 1:
    #            return self.cleaned_data['password1']
    #        raise forms.ValidationError(_("""
    #           password too simple or too short. make it longer or try some combination with
    #           capitals and simple letters, numbers and symbols.
    #        """))

    def clean(self):
        password1 = self.cleaned_data.get('password1')
        password2 = self.cleaned_data.get('password2')
        first_name = self.cleaned_data.get('first_name')
        last_name = self.cleaned_data.get('last_name')
        email = self.cleaned_data.get('email')
        username = self.cleaned_data.get('username')
        account = None
        if email:
            account = email.split('@')[0]

        msg = None
        if password1 and first_name and password1.lower() == first_name.lower():
            msg = _("password must be different to your first name")
        elif password1 and last_name and password1.lower() == last_name.lower():
            msg = _("password must be different to your last name")
        elif password1 and email and password1.lower() == email.lower():
            msg = _("password must be different to your email")
        elif password1 and account and password1.lower() == account.lower():
            msg = _("password can't derive from parts of your email")
        elif password1 and username and password1.lower() == username.lower():
            msg = _("password must be different to your nickname")
        if msg:
            self._errors["password1"] = self.error_class([_(msg)])

        if password1 and password2 and password1 != password2:
            msg = _("submitted passwords doesn't match")
            self._errors["password2"] = self.error_class([msg])
            del self.cleaned_data["password1"]
            del self.cleaned_data["password2"]
        return self.cleaned_data

    def save(self, commit=True):
        # Creo nuovo utente
        new_user = User.objects.create_user(
            username=self.cleaned_data['username'],
            email=self.cleaned_data['email'],
            password=self.cleaned_data['password1'],
        )
        new_user.first_name = self.cleaned_data['first_name']
        new_user.last_name = self.cleaned_data['last_name']
        new_user.is_active = False
        new_user.save()

        # Creo contact
        contact = super(ContactRegistrationForm, self).save(commit=False)
        contact.user = new_user
        contact.save()
        return contact


class RecoveryPasswordForm(PasswordResetForm):
    email = forms.EmailField(
        label=_("e-mail"),
        max_length=150,
        widget=forms.TextInput()
    )

    def save(self, cms_model_instance, request, page_relative_url=None):
        for user in self.users_cache:
            send_recovery_password_email(
                user=user, request=request, cms_model_instance=cms_model_instance, page_relative_url=page_relative_url
            )


class ResetPasswordForm(SetPasswordForm):
    """
    A form that lets a user change his/her password
    """

    def __init__(self, user, *args, **kwargs):
        super(ResetPasswordForm, self).__init__(user, *args, **kwargs)
        self.user = user
        # self.fields['new_password1'].widget.attrs = {'class':'span8'}
        # self.fields['new_password2'].widget.attrs = {'class':'span8'}

    def clean_new_password1(self):
        score = check_password_strenght(self.cleaned_data.get('new_password1'))[0]
        if score > 1:
            return self.cleaned_data['new_password1']

        raise forms.ValidationError(_("your password is weak. please choose a stronger password"))

    def clean(self):
        self.error_messages['notequal_to_firstname'] = _("password must be different to your first name")
        self.error_messages['notequal_to_lastname'] = _("password must be different to your last name")
        self.error_messages['notequal_to_email'] = _("password must be different to your email")
        self.error_messages['notderived_from_email'] = _("password can't derive from parts of your email")
        self.error_messages['password_mismatch'] = _("submitted passwords doesn't match")

        new_password1 = self.cleaned_data.get('new_password1')
        new_password2 = self.cleaned_data.get('new_password2')
        first_name = self.user.first_name
        last_name = self.user.last_name
        email = self.user.email
        if email:
            account = email.split('@')[0]

        msg = None
        if new_password1 and first_name and new_password1.lower() == first_name.lower():
            raise forms.ValidationError(self.error_messages['notequal_to_firstname'])
        if new_password1 and last_name and new_password1.lower() == last_name.lower():
            raise forms.ValidationError(self.error_messages['notequal_to_lastname'])
            # msg = u"La password non puo' essere uguale al tuo Cognome"
        if new_password1 and email and new_password1.lower() == email.lower():
            raise forms.ValidationError(self.error_messages['notequal_to_email'])
            # msg = u"La password non puo' essere uguale alla tua Email"
        if new_password1 and account and new_password1.lower() == account.lower():
            raise forms.ValidationError(self.error_messages['notderived_from_email'])
        if new_password1 and new_password2 and new_password1 != new_password2:
            raise forms.ValidationError(self.error_messages['password_mismatch'])

        return self.cleaned_data


class SetPasswordFormWOldPW(PasswordChangeForm):
    """
    A form that lets a user change his/her password by entering
    their old password.
    """

    def __init__(self, user, *args, **kwargs):
        super(SetPasswordFormWOldPW, self).__init__(user, *args, **kwargs)
        self.user = user
        # self.fields['new_password1'].widget.attrs = {'class':'span8'}
        # self.fields['new_password2'].widget.attrs = {'class':'span8'}
        # self.fields['old_password'].widget.attrs = {'class':'span8'}
