# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from six import python_2_unicode_compatible
from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _

from ckeditor.fields import RichTextField

from jmb.core.db.models import DateModel, UserModel, StatusModel, OrderedModel, DefaultModel

from thux.address.models import Address
from thux.carriage.models import Delivery
from thux.currency.models import Currency
from thux.i18n.models import Language
from thux.office.models import Office
from thux.organization import managers
from thux.payment.models import PaymentForm
from thux.vat.models import Vat


@python_2_unicode_compatible
class OrganizationAddress(Address):
    """
    Extend `address.Address` with 'organization.Organization`
    MODELS:
    :model: `address.Address`
    :model: `organization.Organization`
    """
    organization = models.ForeignKey(
        "Organization",
        db_index=True,
        related_name='addresses',
        verbose_name=_("organization"),
    )
    org_name = models.CharField(
        max_length=100,
        blank=True, null=True,
        db_index=True,
        verbose_name=_("organization name"),
    )
    note = models.TextField(
        blank=True, null=True,
        verbose_name=_("note"),
    )

    class Meta:
        verbose_name = _("organization address")
        verbose_name_plural = _("organization addresses")
        permissions = (
            ("list_organizationaddress", "Can list organization address"),
            ("detail_organizationaddress", "Can detail organization address"),
            ("disable_organizationaddress", "Can disable organization address"),
        )

    def __str__(self):
        return "{organization} {org_name} - {address_type} {name} {number} - {locality} ({province}) - {country}".format(
            organization=self.organization,
            org_name=self.org_name,
            address_type=self.get_address_type_display(),
            name=self.name,
            number=self.number or '',
            locality=self.locality,
            province=self.province,
            country=self.country
        )


@python_2_unicode_compatible
class Organization(DateModel, UserModel, StatusModel, OrderedModel):
    """
    Store organization
    MODELS:
    :model: `office.Office`
    :model: `organization.OrganizationGroup`
    :model: `organization.OrganizationType`
    :model: `organization.OrganizationForm`
    :model: `organization.Field`
    :model: `payment.PaymentForm`
    :model: `currency.Currency`
    :model: `carriage.Delivery`
    :model: `vat.Vat`
    """
    office = models.ForeignKey(
        Office,
        blank=True, null=True,
        db_index=True,
        related_name="organizations",
        verbose_name=_("office"),
    )
    organization_groups = models.ManyToManyField(
        "OrganizationGroup",
        blank=True,
        db_index=True,
        related_name="organizations",
        verbose_name=_("group"),
    )
    organization_type = models.ForeignKey(
        "OrganizationType",
        blank=True, null=True,
        db_index=True,
        related_name="organizations",
        verbose_name=_("organization type"),
    )
    organization_form = models.ForeignKey(
        "OrganizationForm",
        blank=True, null=True,
        db_index=True,
        related_name="organizations",
        verbose_name=_("organization form"),
    )
    field = models.ForeignKey(
        "Field",
        blank=True, null=True,
        db_index=True,
        related_name="organizations",
        verbose_name=_("field"),
    )
    payment_form = models.ForeignKey(
        PaymentForm,
        blank=True, null=True,
        db_index=True,
        related_name="organizations",
        verbose_name=_('payment form'),
    )
    currency = models.ForeignKey(
        Currency,
        blank=True, null=True,
        related_name="organizations",
        db_index=True,
        verbose_name=_("currency"),
    )
    delivery = models.ForeignKey(
        Delivery,
        blank=True, null=True,
        related_name="organizations",
        db_index=True,
        verbose_name=_("delivery"),
    )
    vat = models.ForeignKey(
        Vat,
        blank=True, null=True,
        related_name="organizations",
        db_index=True,
        verbose_name=_("vat"),
    )
    vector = models.ForeignKey(
        "VectorOrganization",
        blank=True, null=True,
        related_name="organizations",
        db_index=True,
        verbose_name=_("vector"),
    )
    code = models.CharField(
        primary_key=True,
        max_length=30,
        verbose_name=_("code"),
    )
    name = models.CharField(
        max_length=100,
        db_index=True,
        verbose_name=_("name"),
    )
    brand = models.CharField(
        max_length=255,
        blank=True, null=True,
        db_index=True,
        verbose_name=_("brand"),
    )
    ssn = models.CharField(
        max_length=16,
        blank=False, null=True,
        db_index=True,
        verbose_name=_("ssn"),
    )
    vat_number = models.CharField(
        max_length=14,
        blank=False, null=True,
        db_index=True,
        verbose_name=_("vat number"),
    )
    url = models.URLField(
        max_length=100,
        blank=True, null=True,
        verbose_name=_("url"),
    )
    email = models.EmailField(
        max_length=100,
        blank=True, null=True,
        verbose_name=_("email"),
    )
    phone = models.CharField(
        max_length=20,
        blank=True, null=True,
        verbose_name=_("phone"),
    )
    phone2 = models.CharField(
        blank=True, null=True,
        max_length=20,
        verbose_name=_("phone2"),
    )
    fax = models.CharField(
        blank=True, null=True,
        max_length=20,
        verbose_name=_("fax"),
    )
    note = RichTextField(
        blank=True,
        verbose_name=_("note"),
    )

    class Meta:
        verbose_name = _("organization")
        verbose_name_plural = _("organizations")
        ordering = ('name',)
        permissions = (
            ("list_organization", "Can list organization"),
            ("detail_organization", "Can detail organization"),
            ("disable_organization", "Can disable organization"),
        )

    def __str__(self):
        return self.name

    def save(self, *args, **kwargs):

        osn = settings.ORGANIZATION_ORGANIZATION_NAME_STYLE
        if osn == "title":
            self.name = self.name.title()
        elif osn == "lower":
            self.name = self.name.lower()
        elif osn == "upper":
            self.name = self.name.upper()

        super(Organization, self).save(*args, **kwargs)

    def get_addresses(self):
        """
        Return all active addresses
        """
        return self.addresses.filter(status=1)

    def get_first_address(self):
        """
        Return first active address
        """
        addresses = self.get_addresses()
        if addresses:
            return self.get_addresses()[0]
        return None

    def get_shipping_addresses(self):
        """
        Return a list of active shipping addresses
        """
        return self.get_addresses().filter(address_scopes__is_shipping_address=True)

    def get_invoice_addresses(self):
        """
        Return a list of active invoice addresses
        """
        return self.get_addresses().filter(address_scopes__is_invoice_address=True)

    @property
    def default_invoice_address1(self):
        """
        Return first active invoice address.
        address1 = `address.type + address.name + address.number`
        """
        addresses = self.get_invoice_addresses()
        if addresses:
            address = addresses[0]
            return "{address} {name}, {number}".format(
                address=address.get_address_type_display() or "",
                name=address.name,
                number=address.number,
            )
        return None

    @property
    def default_invoice_address2(self):
        """
        Returns first active invoice address.
        address2 = `address.zip_code + address.locality + address.province`
        """
        addresses = self.get_invoice_addresses()
        if addresses:
            address = addresses[0]
            return "{zipcode} {locality} ({province})".format(
                zipcode=address.zip_code,
                locality=address.locality,
                province=address.province,
            )
        return None

    @property
    def default_invoice_address3(self):
        """
        Returns first active invoice address.
        address3 = `address.country`
        """
        addresses = self.get_invoice_addresses()
        if addresses:
            address = addresses[0]
            return "{country}".format(
                country=address.country,
            )
        return None

    @property
    def default_shipping_address1(self):
        """
        Returns first active shipping address.
        address1 = `address.type + address.name + address.number`
        """
        addresses = self.get_shipping_addresses()
        if addresses:
            address = addresses[0]
            return "{address} {name}, {number}".format(
                address=address.get_address_type_display() or "",
                name=address.name,
                number=address.number,
            )
        return None

    @property
    def default_shipping_address2(self):
        """
        Returns first active shipping address.
        address2 = `address.zip_code + address.locality + address.province`
        """
        addresses = self.get_shipping_addresses()
        if addresses:
            address = addresses[0]
            return "{zipcode} {locality} ({province})".format(
                zipcode=address.zip_code,
                locality=address.locality,
                province=address.province,
            )
        return None

    @property
    def default_shipping_address3(self):
        """
        Returns first active shipping address.
        address3 = `address.country`
        """
        addresses = self.get_shipping_addresses()
        if addresses:
            address = addresses[0]
            return "{country}".format(
                country=address.country,
            )
        return None

    def get_contacts(self, other_filters=dict()):
        """
        Returns all active contacts
        """
        contacts = self.contacts.filter(status=1)
        if other_filters:
            contacts = contacts.filter(**other_filters)
        return contacts

    def get_default_contact(self):
        """
        Returns default contact: 'contact.default=True'
        """
        contact_filter = {'default': True}
        contacts = self.get_contacts(other_filters=contact_filter)
        if contacts:
            return contacts[0]
        return None

#    def get_quotation_contacts(self):
#        """
#        Returns all active contacts with 'contact.send_quotation_by_email=True'
#        """
#        contact_filter = {'send_quotation_by_email':True}
#        return self.get_contacts(other_filters=contact_filter)
#
#    def get_procedure_contacts(self):
#        """
#        Returns all active contacts with 'contact.send_procedure_by_email=True'
#        """
#        contact_filter = {'send_procedure_by_email':True}
#        return self.get_contacts(other_filters=contact_filter)
#
#
#    def get_ddt_contacts(self):
#        """
#        Returns all active contacts with 'contact.send_ddt_by_email=True'
#        """
#        contact_filter = {'send_ddt_by_email':True}
#        return self.get_contacts(other_filters=contact_filter)
#
#
#    def get_invoice_contacts(self):
#        """
#        Returns all active contacts with 'contact.send_invoice_by_email=True'
#        """
#        contact_filter = {'send_invoice_by_email':True}
#        return self.get_contacts(other_filters=contact_filter)
#
#
#    def get_all_mails_for_procedure(self):
#        """
#        Returns all contact mails with 'send_procedure_by_email=True'
#        """
#        contact_filter = {'send_procedure_by_email':True}
#        return self.get_all_mails(contact_filter=contact_filter)
#
#
#    def get_all_mails_for_ddt(self):
#        """
#        Returns all contact mails with 'send_ddt_by_email=True'
#        """
#        contact_filter = {'send_ddt_by_email':True}
#        return self.get_all_mails(contact_filter=contact_filter)
#
#
#    def get_all_mails_for_invoice(self):
#        """
#        Returns all contact mails with 'send_invoice_by_email=True'
#        """
#        contact_filter = {'send_invoice_by_email':True}
#        return self.get_all_mails(contact_filter=contact_filter)
#
#    def get_all_mails_for_quotation(self):
#        """
#        Returns all contact mails with 'send_quotation_by_email=True'
#        """
#        contact_filter = {'send_quotation_by_email':True}
#        return self.get_all_mails(contact_filter=contact_filter)
#
#    def get_all_mails(self, contact_filter={}):
#        """
#        Returns organization email, contacts emails and agents email.
#        Example:
#        (
#            ('info@thundersystems.it (Organization)', 'info@thundersystems.it'),
#            ('vparenti@thundersystems.it (Contact)', 'vparenti@thundersystems.it'),
#            …
#        )
#        """
#        mails = []
#
#        # Organization email
#        if self.email:
#            label_mail = ugettext("{email} (Organization)".format(
#                email=self.email,
#            ))
#            choice_mail = (self.email, label_mail)
#            if choice_mail not in mails:
#                mails.append(choice_mail)
#
#        # All contacts emails
#        for c in self.get_contacts(other_filters=contact_filter):
#            if c.email:
#                label_mail = ugettext("{email} (Contact)".format(
#                    email=c.email,
#                ))
#                choice_mail = (c.email, label_mail)
#                if choice_mail not in mails:
#                    mails.append(choice_mail)
#
#        # All agents emails
#        for a in self.get_agents():
#            if a.email:
#                label_mail = ugettext("{email} (Agent)".format(
#                    email=a.email,
#                ))
#                choice_mail = (a.email, label_mail)
#                if choice_mail not in mails:
#                    mails.append(choice_mail)
#
#        return mails


@python_2_unicode_compatible
class EmailTypology(UserModel, DateModel, StatusModel, OrderedModel):
    code = models.CharField(
        primary_key=True,
        max_length=255,
        verbose_name=_('code'),
    )
    name = models.CharField(
        max_length=255,
        unique=True,
        verbose_name=_('name'),
    )
    slug = models.SlugField(
        max_length=255,
        unique=True,
        verbose_name=_('slug'),
    )

    class Meta:
        permissions = (
            ("list_emailtypology", "Can list email typology"),
            ("detail_emailtypology", "Can detail email typology"),
            ("disable_emailtypology", "Can disable email typology"),
        )
        verbose_name = _("email typology")
        verbose_name_plural = _("email typologies")
        ordering = ('ordering', 'name')

    def __str__(self):
        return u'{}'.format(self.name)


@python_2_unicode_compatible
class PhoneTypology(UserModel, DateModel, StatusModel, OrderedModel):
    code = models.CharField(
        primary_key=True,
        max_length=255,
        verbose_name=_('code'),
    )
    name = models.CharField(
        max_length=255,
        unique=True,
        verbose_name=_('name'),
    )
    slug = models.SlugField(
        max_length=255,
        unique=True,
        verbose_name=_('slug'),
    )

    class Meta:
        permissions = (
            ("list_phonetypology", "Can list phone typology"),
            ("detail_phonetypology", "Can detail phone typology"),
            ("disable_phonetypology", "Can disable phone typology"),
        )
        verbose_name = _("phone typology")
        verbose_name_plural = _("phone typologies")
        ordering = ('ordering', 'name')

    def __str__(self):
        return u'{}'.format(self.name)


@python_2_unicode_compatible
class OwnerOrganization(Organization):
    objects = managers.OwnerOrganizationManager()

    class Meta:
        proxy = True
        verbose_name = _("owner organization")
        verbose_name_plural = _("owner organizations")
        permissions = (
            ("list_ownerorganization", "Can list owner organization"),
            ("detail_ownerorganization", "Can detail owner organization"),
            ("disable_ownerorganization", "Can disable owner organization"),
        )

    def __str__(self):
        return self.name


@python_2_unicode_compatible
class CustomerOrganization(Organization):
    objects = managers.CustomerOrganizationManager()

    class Meta:
        proxy = True
        verbose_name = _("customer organization")
        verbose_name_plural = _("customer organizations")
        permissions = (
            ("list_customerorganization", "Can list customer organization"),
            ("detail_customerorganization", "Can detail customer organization"),
            ("disable_customerorganization", "Can disable customer organization"),
        )

    def __str__(self):
        return self.name


@python_2_unicode_compatible
class SupplierOrganization(Organization):
    objects = managers.SupplierOrganizationManager()

    class Meta:
        proxy = True
        verbose_name = _("supplier organization")
        verbose_name_plural = _("supplier organizations")
        permissions = (
            ("list_supplierorganization", "Can list supplier organization"),
            ("detail_supplierorganization", "Can detail supplier organization"),
            ("disable_supplierorganization", "Can disable supplier organization"),
        )

    def __str__(self):
        return self.name


@python_2_unicode_compatible
class VectorOrganization(Organization):
    objects = managers.VectorOrganizationManager()

    class Meta:
        proxy = True
        verbose_name = _("vector organization")
        verbose_name_plural = _("vector organizations")
        permissions = (
            ("list_vectororganization", "Can list vector organization"),
            ("detail_vectororganization", "Can detail vector organization"),
            ("disable_vectororganization", "Can disable vector organization"),
        )

    def __str__(self):
        return self.name


@python_2_unicode_compatible
class OrganizationEmail(DateModel, UserModel, StatusModel, DefaultModel, OrderedModel):
    organization = models.ForeignKey(
        Organization,
        related_name='emails',
        verbose_name=_('organization'),
    )
    typology = models.ForeignKey(
        EmailTypology,
        related_name='organization_emails',
        verbose_name=_('typology'),
    )
    email = models.EmailField(
        max_length=255,
        verbose_name=_("email"),
    )

    class Meta:
        verbose_name = _("organization email")
        verbose_name_plural = _("organization emails")
        unique_together = (('organization', 'email',),)
        permissions = (
            ("list_organizationemail", "Can list organization email"),
            ("detail_organizationemail", "Can detail organization email"),
            ("disable_organizationemail", "Can disable organization email"),
        )

    def __str__(self):
        return u'{}'.format(self.email)


class OwnerOrganizationEmail(OrganizationEmail):
    objects = managers.OwnerOrganizationEmailManager()

    class Meta:
        proxy = True
        verbose_name = _("owner organization")
        verbose_name_plural = _("owner organizations")
        permissions = (
            ("list_ownerorganizationemail", "Can list owner organization email"),
            ("detail_ownerorganizationemail", "Can detail owner organization email"),
            ("disable_ownerorganizationemail", "Can disable owner organization email"),
        )


class CustomerOrganizationEmail(OrganizationEmail):
    objects = managers.CustomerOrganizationEmailManager()

    class Meta:
        proxy = True
        verbose_name = _("customer organization")
        verbose_name_plural = _("customer organizations")
        permissions = (
            ("list_customerorganizationemail", "Can list customer organization email"),
            ("detail_customerorganizationemail", "Can detail customer organization email"),
            ("disable_customerorganizationemail", "Can disable customer organization email"),
        )


class SupplierOrganizationEmail(OrganizationEmail):
    objects = managers.SupplierOrganizationEmailManager()

    class Meta:
        proxy = True
        verbose_name = _("supplier organization")
        verbose_name_plural = _("supplier organizations")
        permissions = (
            ("list_supplierorganizationemail", "Can list supplier organization email"),
            ("detail_supplierorganizationemail", "Can detail supplier organization email"),
            ("disable_supplierorganizationemail", "Can disable supplier organization email"),
        )


class VectorOrganizationEmail(OrganizationEmail):
    objects = managers.VectorOrganizationEmailManager()

    class Meta:
        proxy = True
        verbose_name = _("vector organization")
        verbose_name_plural = _("vector organizations")
        permissions = (
            ("list_vectororganizationemail", "Can list vector organization email"),
            ("detail_vectororganizationemail", "Can detail vector organization email"),
            ("disable_vectororganizationemail", "Can disable vector organization email"),
        )


@python_2_unicode_compatible
class OrganizationGroup(DateModel, UserModel, StatusModel, OrderedModel):
    """
    Store organization's groups
    """
    code = models.CharField(
        primary_key=True,
        max_length=15,
        verbose_name=_("code"),
    )
    name = models.CharField(
        unique=True,
        max_length=100,
        db_index=True,
        verbose_name=_("name"),
    )
    description = models.TextField(
        blank=True, null=True,
        verbose_name=_("description"),
    )

    class Meta:
        verbose_name = _("organization group")
        verbose_name_plural = _("organization groups")
        ordering = ("ordering", "name")
        permissions = (
            ("list_organizationgroup", "Can list organization group"),
            ("detail_organizationgroup", "Can detail organization group"),
            ("disable_organizationgroup", "Can disable organization group"),
        )

    def __str__(self):
        return self.name


@python_2_unicode_compatible
class OrganizationForm(DateModel, UserModel, StatusModel, OrderedModel):
    """
    Store organization's forms
    EXAMPLE: 'SpA', 'Srl', 'Ltd', 'Plc', 'Inc'
    """
    code = models.CharField(
        primary_key=True,
        max_length=15,
        verbose_name=_("code"),
    )
    name = models.CharField(
        unique=True,
        max_length=100,
        db_index=True,
        verbose_name=_("name"),
    )
    description = models.TextField(
        blank=True, null=True,
        verbose_name=_("description"),
    )

    class Meta:
        verbose_name = _("organization form")
        verbose_name_plural = _("organization forms")
        ordering = ("ordering", "name")
        permissions = (
            ("list_organizationform", "Can list organization form"),
            ("detail_organizationform", "Can detail organization form"),
            ("disable_organizationform", "Can disable organization form"),
        )

    def __str__(self):
        return self.name


@python_2_unicode_compatible
class OrganizationType(DateModel, UserModel, StatusModel, OrderedModel):
    """
    Stores organization's types
    EXAMPLE: 'end user', 'retailer', 'dealer'
    """
    code = models.CharField(
        primary_key=True,
        max_length=15,
        verbose_name=_("code"),
    )
    category = models.CharField(
        max_length=15,
        choices=settings.ORGANIZATION_ORGANIZATIONTYPE_DEF,
        db_index=True,
        verbose_name=_("category"),
    )
    name = models.CharField(
        unique=True,
        max_length=100,
        db_index=True,
        verbose_name=_("name"),
    )
    description = models.TextField(
        blank=True, null=True,
        verbose_name=_("description"),
    )
    note = models.TextField(
        blank=True, null=True,
        verbose_name=_("note"),
    )

    class Meta:
        verbose_name = _("organization type")
        verbose_name_plural = _("organization types")
        ordering = ('ordering', 'name')
        permissions = (
            ("list_organizationtype", "Can list organization type"),
            ("detail_organizationtype", "Can detail organization type"),
            ("disable_organizationtype", "Can disable organization type"),
        )

    def __str__(self):
        return self.name


@python_2_unicode_compatible
class Field(DateModel, UserModel, StatusModel, OrderedModel):
    """
    Stores organization's fields
    EXAMPLE: 'Entertainment', 'Financial'
    """
    code = models.CharField(
        primary_key=True,
        max_length=15,
        verbose_name=_("code"),
    )
    name = models.CharField(
        unique=True,
        max_length=100,
        db_index=True,
        verbose_name=_("name"),
    )
    description = models.TextField(
        blank=True, null=True,
        verbose_name=_("description"),
    )

    class Meta:
        verbose_name = _("field")
        verbose_name_plural = _("fields")
        ordering = ("ordering", "name")
        permissions = (
            ("list_field", "Can list field"),
            ("detail_field", "Can detail field"),
            ("disable_field", "Can disable field"),
        )

    def __str__(self):
        return self.name


@python_2_unicode_compatible
class ContactRole(DateModel, UserModel, StatusModel, OrderedModel):
    """
    Stores contact roles
    EXAMPLE: 'sales manager', 'development manager'
    """
    code = models.CharField(
        primary_key=True,
        max_length=15,
        verbose_name=_("code"),
    )
    name = models.CharField(
        unique=True,
        max_length=100,
        db_index=True,
        verbose_name=_("name"),
    )

    class Meta:
        verbose_name = _("contact role")
        verbose_name_plural = _("contact roles")
        ordering = ("ordering", "name")
        permissions = (
            ("list_contactrole", "Can list contact role"),
            ("detail_contactrole", "Can detail contact role"),
            ("disable_contactrole", "Can disable contact role"),
        )

    def __str__(self):
        return self.name


@python_2_unicode_compatible
class ContactTitle(DateModel, UserModel, StatusModel, OrderedModel):
    """
    Stores contact titles
    EXAMPLE: 'mr. mister', 'dr. doctor'
    """
    code = models.CharField(
        primary_key=True,
        max_length=15,
        verbose_name=_("code"),
    )
    name = models.CharField(
        unique=True,
        max_length=100,
        db_index=True,
        verbose_name=_("name"),
    )
    abbreviation = models.CharField(
        unique=True,
        max_length=10,
        verbose_name=_("abbreviation"),
    )

    class Meta:
        verbose_name = _("contact title")
        verbose_name_plural = _("contact titles")
        ordering = ("ordering", "name")
        permissions = (
            ("list_contacttitle", "Can list contact title"),
            ("detail_contacttitle", "Can detail contact title"),
            ("disable_contacttitle", "Can disable contact title"),
        )

    def __str__(self):
        return self.abbreviation


@python_2_unicode_compatible
class Contact(DateModel, UserModel, StatusModel, DefaultModel, OrderedModel):
    """
    Stores contacts
    MODELS:
    :model: `organization.Organization`
    :model: `auth.User`
    :model: `organization.ContactTitle`
    :model: `organization.ContactRole`
    :model: `i18n.Language`
    """

    objects = managers.ContactManager()

    code = models.CharField(
        primary_key=True,
        max_length=15,
        verbose_name=_("code"),
    )
    user = models.OneToOneField(
        settings.AUTH_USER_MODEL,
        blank=True, null=True,
        db_index=True,
        related_name='contact',
        verbose_name=_("user"),
    )
    organization = models.ForeignKey(
        Organization,
        blank=True, null=True,
        db_index=True,
        related_name='contacts',
        verbose_name=_("organization"),
    )
    title = models.ForeignKey(
        ContactTitle,
        blank=True, null=True,
        related_name='contacts',
        verbose_name=_("title"),
    )
    role = models.ForeignKey(
        ContactRole,
        blank=True, null=True,
        related_name='contacts',
        verbose_name=_("role"),
    )
    language = models.ForeignKey(
        Language,
        blank=True, null=True,
        db_index=True,
        verbose_name=_('language'),
        help_text=_('language'),
    )
    last_name = models.CharField(
        max_length=100,
        null=True, blank=True,
        verbose_name=_("last name"),
    )
    first_name = models.CharField(
        max_length=100,
        blank=True, null=True,
        verbose_name=_("first name"),
    )
    phone = models.CharField(
        max_length=20,
        verbose_name=_("phone"),
        blank=True, null=True
    )
    mobile = models.CharField(
        max_length=20,
        blank=True, null=True,
        verbose_name=_("mobile"),
    )
    fax = models.CharField(
        max_length=20,
        blank=True, null=True,
        verbose_name=_("fax"),
    )
    email = models.EmailField(
        max_length=100,
        blank=True, null=True,
        verbose_name=_("email"),
    )
    note = models.TextField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_("note"),
    )
    in_addressbook = models.NullBooleanField(
        blank=True, null=True,
        verbose_name=_("in addressbook"),
        help_text=_("if this number is in the phone address book.")
    )
    # se i campi successivi sono 'flaggati' il campo 'email' diventa obbligatorio
    send_order_by_email = models.BooleanField(
        default=False,
        verbose_name=_("send procedure by email"),
    )
    send_ddt_by_email = models.BooleanField(
        default=False,
        verbose_name=_("send DDT by email"),
    )
    send_invoice_by_email = models.BooleanField(
        default=False,
        verbose_name=_("send invoice by email"),
    )
    send_quotation_by_email = models.BooleanField(
        default=False,
        verbose_name=_("send quotation by email"),
    )
    photo = models.ImageField(
        upload_to=settings.ORGANIZATION_CONTACT_UPLOAD_PHOTO,
        blank=True, null=True,
        verbose_name=_("photo"),
    )
    contact_groups = models.ManyToManyField(
        "OrganizationGroup",
        blank=True,
        db_index=True,
        related_name="contacts",
        verbose_name=_("group"),
    )
    is_worker = models.BooleanField(
        default=False,
        verbose_name=_("is worker"),
    )

    class Meta:
        verbose_name = _("contact")
        verbose_name_plural = _("contacts")
        ordering = ['ordering', 'last_name', 'first_name']
        permissions = (
            ("list_contact", "Can list contact"),
            ("detail_contact", "Can detail contact"),
            ("disable_contact", "Can disable contact"),
        )

    def __str__(self):
        txt = "{title} {last_name} {first_name}".format(
            title=self.title or '',
            last_name=self.last_name,
            first_name=self.first_name,
        )
        if self.role:
            txt += " ({role})".format(
                role=self.role,
            )
        return txt

    def save(self, **kw):
        osn = settings.ORGANIZATION_CONTACT_NAME_STYLE
        if osn == "title":
            self.last_name = self.last_name.title()
            self.first_name = self.first_name.title()
        elif osn == "lower":
            self.last_name = self.last_name.lower()
            self.first_name = self.first_name.lower()
        elif osn == "upper":
            self.last_name = self.last_name.upper()
            self.first_name = self.first_name.upper()

        super(Contact, self).save(**kw)

    def get_full_name(self):
        """
        returns the contact's full name (last name and first name)
        """
        return "{last_name} {first_name}".format(
            last_name=self.last_name,
            first_name=self.first_name,
        )

    full_name = property(get_full_name)

    def get_signature(self):
        """
        returns the contact's full name (first name and last name)
        """
        return "{first_name} {last_name}".format(
            first_name=self.first_name,
            last_name=self.last_name,
        )
    signature = property(get_signature)

    def get_email(self):
        """Return an email eather in contact or user if any"""
        try:
            return self.email or self.user.email
        except AttributeError:  # no user connected
            pass


@python_2_unicode_compatible
class OwnerContact(Contact):
    objects = managers.OwnerContactManager()

    class Meta:
        proxy = True
        verbose_name = _("owner contact")
        verbose_name_plural = _("owner contacts")
        permissions = (
            ("list_ownercontact", "Can list owner contact"),
            ("detail_ownercontact", "Can detail owner contact"),
            ("disable_ownercontact", "Can disable owner contact"),
        )

    def __str__(self):
        txt = "{title} {last_name} {first_name}".format(
            title=self.title or '',
            last_name=self.last_name,
            first_name=self.first_name,
        )
        if self.role:
            txt += " ({role})".format(
                role=self.role,
            )
        return txt


@python_2_unicode_compatible
class CustomerContact(Contact):
    objects = managers.CustomerContactManager()

    class Meta:
        proxy = True
        verbose_name = _("customer contact")
        verbose_name_plural = _("customer contacts")
        permissions = (
            ("list_customercontact", "Can list customer contact"),
            ("detail_customercontact", "Can detail customer contact"),
            ("disable_customercontact", "Can disable customer contact"),
        )

    def __str__(self):
        txt = "{title} {last_name} {first_name}".format(
            title=self.title or '',
            last_name=self.last_name,
            first_name=self.first_name,
        )
        if self.role:
            txt += " ({role})".format(
                role=self.role,
            )
        return txt


@python_2_unicode_compatible
class SupplierContact(Contact):
    objects = managers.SupplierContactManager()

    class Meta:
        proxy = True
        verbose_name = _("supplier contact")
        verbose_name_plural = _("supplier contacts")
        permissions = (
            ("list_suppliercontact", "Can list supplier contact"),
            ("detail_suppliercontact", "Can detail supplier contact"),
            ("disable_suppliercontact", "Can disable supplier contact"),
        )

    def __str__(self):
        txt = "{title} {last_name} {first_name}".format(
            title=self.title or '',
            last_name=self.last_name,
            first_name=self.first_name,
        )
        if self.role:
            txt += " ({role})".format(
                role=self.role,
            )
        return txt


@python_2_unicode_compatible
class WorkerContact(Contact):
    objects = managers.WorkerContactManager()

    class Meta:
        proxy = True
        verbose_name = _("worker contact")
        verbose_name_plural = _("worker contacts")
        permissions = (
            ("list_workercontact", "Can list worker contact"),
            ("detail_workercontact", "Can detail worker contact"),
            ("disable_workercontact", "Can disable worker contact"),
        )

    def __str__(self):
        txt = "{title} {last_name} {first_name}".format(
            title=self.title or '',
            last_name=self.last_name,
            first_name=self.first_name,
        )
        if self.role:
            txt += " ({role})".format(
                role=self.role,
            )
        return txt


@python_2_unicode_compatible
class VectorContact(Contact):
    objects = managers.VectorContactManager()

    class Meta:
        proxy = True
        verbose_name = _("vector contact")
        verbose_name_plural = _("vector contacts")
        permissions = (
            ("list_vectorcontact", "Can list vector contact"),
            ("detail_vectorcontact", "Can detail vector contact"),
            ("disable_vectorcontact", "Can disable vector contact"),
        )

    def __str__(self):
        txt = "{title} {last_name} {first_name}".format(
            title=self.title or '',
            last_name=self.last_name,
            first_name=self.first_name,
        )
        if self.role:
            txt += " ({role})".format(
                role=self.role,
            )
        return txt


@python_2_unicode_compatible
class ContactEmail(DateModel, UserModel, StatusModel, DefaultModel, OrderedModel):
    contact = models.ForeignKey(
        Contact,
        related_name='emails',
        verbose_name=_('contact'),
    )
    typology = models.ForeignKey(
        EmailTypology,
        related_name='contact_emails',
        verbose_name=_('typology'),
    )
    email = models.EmailField(
        max_length=255,
        verbose_name=_("email"),
    )

    class Meta:
        verbose_name = _("contact email")
        verbose_name_plural = _("contact emails")
        unique_together = (('contact', 'email',),)
        permissions = (
            ("list_contactemail", "Can list contact email"),
            ("detail_contactemail", "Can detail contact email"),
            ("disable_contactemail", "Can disable contact email"),
        )

    def __str__(self):
        return u'{}'.format(self.email)


class OwnerContactEmail(ContactEmail):
    objects = managers.OwnerContactEmailManager()

    class Meta:
        proxy = True
        verbose_name = _("owner contact")
        verbose_name_plural = _("owner contacts")
        permissions = (
            ("list_ownercontactemail", "Can list owner contact email"),
            ("detail_ownercontactemail", "Can detail owner contact email"),
            ("disable_ownercontactemail", "Can disable owner contact email"),
        )


class CustomerContactEmail(ContactEmail):
    objects = managers.CustomerContactEmailManager()

    class Meta:
        proxy = True
        verbose_name = _("customer contact")
        verbose_name_plural = _("customer contacts")
        permissions = (
            ("list_customercontactemail", "Can list customer contact email"),
            ("detail_customercontactemail", "Can detail customer contact email"),
            ("disable_customercontactemail", "Can disable customer contact email"),
        )


class SupplierContactEmail(ContactEmail):
    objects = managers.SupplierContactEmailManager()

    class Meta:
        proxy = True
        verbose_name = _("supplier contact")
        verbose_name_plural = _("supplier contacts")
        permissions = (
            ("list_suppliercontactemail", "Can list supplier contact email"),
            ("detail_suppliercontactemail", "Can detail supplier contact email"),
            ("disable_suppliercontactemail", "Can disable supplier contact email"),
        )


class WorkerContactEmail(ContactEmail):
    objects = managers.WorkerContactEmailManager()

    class Meta:
        proxy = True
        verbose_name = _("worker contact")
        verbose_name_plural = _("worker contacts")
        permissions = (
            ("list_workercontactemail", "Can list worker contact email"),
            ("detail_workercontactemail", "Can detail worker contact email"),
            ("disable_workercontactemail", "Can disable worker contact email"),
        )


class VectorContactEmail(ContactEmail):
    objects = managers.VectorContactEmailManager()

    class Meta:
        proxy = True
        verbose_name = _("vector contact")
        verbose_name_plural = _("vector contacts")
        permissions = (
            ("list_vectorcontactemail", "Can list vector contact email"),
            ("detail_vectorcontactemail", "Can detail vector contact email"),
            ("disable_vectorcontactemail", "Can disable vector contact email"),
        )


@python_2_unicode_compatible
class ContactPhone(DateModel, UserModel, StatusModel, DefaultModel, OrderedModel):
    contact = models.ForeignKey(
        Contact,
        related_name='phones',
        verbose_name=_('contact'),
    )
    typology = models.ForeignKey(
        PhoneTypology,
        related_name='contact_phones',
        verbose_name=_('typology'),
    )
    phone = models.CharField(
        max_length=20,
        verbose_name=_("phone"),
    )

    class Meta:
        verbose_name = _("contact phone")
        verbose_name_plural = _("contact phones")
        unique_together = (('contact', 'phone',),)
        permissions = (
            ("list_contactphone", "Can list contact phone"),
            ("detail_contactphone", "Can detail contact phone"),
            ("disable_contactphone", "Can disable contact phone"),
        )

    def __str__(self):
        return u'{}'.format(self.phone)


class OwnerContactPhone(ContactPhone):
    objects = managers.OwnerContactPhoneManager()

    class Meta:
        proxy = True
        verbose_name = _("owner contact")
        verbose_name_plural = _("owner contacts")
        permissions = (
            ("list_ownercontactphone", "Can list owner contact phone"),
            ("detail_ownercontactphone", "Can detail owner contact phone"),
            ("disable_ownercontactphone", "Can disable owner contact phone"),
        )


class CustomerContactPhone(ContactPhone):
    objects = managers.CustomerContactPhoneManager()

    class Meta:
        proxy = True
        verbose_name = _("customer contact")
        verbose_name_plural = _("customer contacts")
        permissions = (
            ("list_customercontactphone", "Can list customer contact phone"),
            ("detail_customercontactphone", "Can detail customer contact phone"),
            ("disable_customercontactphone", "Can disable customer contact phone"),
        )


class SupplierContactPhone(ContactPhone):
    objects = managers.SupplierContactPhoneManager()

    class Meta:
        proxy = True
        verbose_name = _("supplier contact")
        verbose_name_plural = _("supplier contacts")
        permissions = (
            ("list_suppliercontactphone", "Can list supplier contact phone"),
            ("detail_suppliercontactphone", "Can detail supplier contact phone"),
            ("disable_suppliercontactphone", "Can disable supplier contact phone"),
        )


class WorkerContactPhone(ContactPhone):
    objects = managers.WorkerContactPhoneManager()

    class Meta:
        proxy = True
        verbose_name = _("worker contact")
        verbose_name_plural = _("worker contacts")
        permissions = (
            ("list_workercontactphone", "Can list worker contact phone"),
            ("detail_workercontactphone", "Can detail worker contact phone"),
            ("disable_workercontactphone", "Can disable worker contact phone"),
        )


class VectorContactPhone(ContactPhone):
    objects = managers.VectorContactPhoneManager()

    class Meta:
        proxy = True
        verbose_name = _("vector contact")
        verbose_name_plural = _("vector contacts")
        permissions = (
            ("list_vectorcontactphone", "Can list vector contact phone"),
            ("detail_vectorcontactphone", "Can detail vector contact phone"),
            ("disable_vectorcontactphone", "Can disable vector contact phone"),
        )
